from django.db import models
from rest import models as rm
from rest.encryption import DECRYPTER, ENCRYPTER
from rest import UberDict

SETTINGS_CACHE = UberDict()


class Settings(models.Model, rm.RestModel, rm.MetaDataModel):
    """
    Create collection of settings.
    This is very useful for create special blocks of settings.
    """
    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)

    name = models.CharField(max_length=128, unique=True)

    owner = models.ForeignKey(
        "account.Member", related_name="settings",
        null=True, default=None, on_delete=models.CASCADE)
    group = models.ForeignKey(
        "account.Group", related_name="settings",
        null=True, default=None, on_delete=models.CASCADE)

    class RestMeta:
        GRAPHS = {
            "default": {
                "extra": ["metadata"],
            }
        }


class SettingsMetaData(rm.MetaDataBase):
    parent = models.ForeignKey(Settings, related_name="properties", on_delete=models.CASCADE)


def setSetting(name, key, value, encrypt=False):
    obj = Settings.objects.filter(name=name).last()
    if obj is None:
        obj = Settings(name=name)
        obj.save()
    if encrypt:
        value = ENCRYPTER.encrypt(value)
    obj.setProperty(key, value)
    SETTINGS_CACHE[name] = UberDict.fromdict(obj.metadata())


def getSetting(name, key, decrypt=False, cache=True):
    settings = getSettingsDict(name)
    if settings is None:
        return None
    if not decrypt:
        return settings.get(key, None)

    value = settings.get(f"decrypted.{key}", None)
    if value is None:
        return value

    value = DECRYPTER.decrypt(value)
    if cache:
        settings.set(f"decrypted.{key}", value)
    return value


def getSettings(name):
    return Settings.objects.filter(name=name).last()


def getSettingsDict(name, refresh=False):
    if refresh or name not in SETTINGS_CACHE:
        obj = Settings.objects.filter(name=name).last()
        if obj:
            SETTINGS_CACHE[name] = UberDict.fromdict(obj.metadata())
    return SETTINGS_CACHE.get(name, None)



# Generated by Django 4.1.4 on 2023-01-06 22:20

from django.db import migrations
import rest.fields


class Migration(migrations.Migration):

    dependencies = [
        ('incident', '0006_delete_eventmetadata'),
    ]

    operations = [
        migrations.AddField(
            model_name='event',
            name='metadata',
            field=rest.fields.JSONField(default=None, null=True),
        ),
    ]



# coding: utf-8

"""
    mistune_contrib.meta
    ~~~~~~~~~~~~~~~~~~~~

    Support Meta features for mistune. Metadata are keywords headers at the
    top of the Markdown text:

        Title: A Metadata DEMO
        Author: Hsiaoming Yang

    :copyright: (c) 2015 by Hsiaoming Yang.
"""


import re


INDENTATION = re.compile(r'\n\s{2,}')
META = re.compile(r'^(\w+):\s*(.*(?:\n\s{2,}.*)*)\n')


def parse(text):
    """Parse the given text into metadata and strip it for a Markdown parser.

    :param text: text to be parsed
    """
    rv = {}
    m = META.match(text)

    while m:
        key = m.group(1)
        value = m.group(2)
        value = INDENTATION.sub('\n', value.strip())
        rv[key] = value
        text = text[len(m.group(0)):]
        m = META.match(text)

    return rv, text



from rest.decorators import periodic
from taskqueue.models import Task


# run every day at 9am UTC
# @periodic(hour=9)
# def run_background_task(force=False, verbose=False, now=None):
#     # this will call a method in tq.py called run_example using a async task
#     Task.Publish("wiki", "run_example", channel="tq_app_handler")





from django.db import models
from rest import models as rm


class Mailbox(models.Model, rm.RestModel, rm.MetaDataModel):
    class RestMeta:
        DEFAULT_SORT = "-id"

    created = models.DateTimeField(auto_now_add=True, editable=False, db_index=True)
    modified = models.DateTimeField(auto_now=True)
    email = models.CharField(max_length=255, db_index=True)
    state = models.IntegerField(default=0, db_index=True)
    # define how this email address should be handeled 
    tq_app = models.CharField(max_length=255, null=True, default=None)
    tq_handler = models.CharField(max_length=255, null=True, default=None)
    tq_channel = models.CharField(max_length=255, null=True, default=None)

    def __str__(self):
        return f"inbox:{self.email}.{self.tq_app}.{self.tq_handler}.{self.tq_channel}"


class MailboxMetaData(rm.MetaDataBase):
    parent = models.ForeignKey(Mailbox, related_name="properties", on_delete=models.CASCADE)


class Message(models.Model, rm.RestModel):
    class RestMeta:
        CAN_SAVE = CAN_CREATE = False
        CAN_DELETE = True
        DEFAULT_SORT = "-id"

    created = models.DateTimeField(auto_now_add=True, editable=False, db_index=True)
    modified = models.DateTimeField(auto_now=True)
    sent_at = models.DateTimeField()

    state = models.IntegerField(default=0, db_index=True)

    to_email = models.CharField(max_length=255, db_index=True)
    to_name = models.CharField(max_length=255, null=True, default=None)
    to = models.TextField()
    cc = models.TextField(null=True, default=None)

    from_email = models.CharField(max_length=255, db_index=True)
    from_name = models.CharField(max_length=255, null=True, default=None)
    
    subject = models.CharField(max_length=255, null=True, default=None)
    message = models.TextField(null=True, default=None)

    body = models.TextField(null=True, default=None)
    html = models.TextField(null=True, default=None)

    def __str__(self):
        return f"message: to:{self.to_email} from:{self.from_email} subject: {self.subject}"


class Attachment(models.Model, rm.RestModel):
    class RestMeta:
        CAN_SAVE = CAN_CREATE = False
        DEFAULT_SORT = "-id"
        GRAPHS = {
            "default": {
                "graphs": {
                    "media": "basic"
                },
            }
        }
    created = models.DateTimeField(auto_now_add=True, editable=False, db_index=True)
    name = models.CharField(max_length=255, null=True, default=None)
    content_type = models.CharField(max_length=128, null=True, default=None)
    message = models.ForeignKey(Message, related_name="attachments", on_delete=models.CASCADE)
    media = models.ForeignKey("medialib.MediaItem", related_name="attachments", on_delete=models.CASCADE)

    def __str__(self):
        return f"attachment: to:{self.message.to_email} from:{self.message.from_email} filename: {self.name}"



import re

from django.db.models import Q
from .datem import parseDateTime, parseDate
from rest import helpers
from datetime import datetime, timedelta

SEARCH_COMPARES = [
    "==",
    "<=",
    ">=",
    "!=",
    ">",
    "<",
    ":",
    "=",
]

COMPARATOR_TO_FILTER = {
    ":": "icontains",
    "=":"icontains",
    "==":"iexact",
    "<":"lt",
    "<=":"lte",
    ">":"gt",
    ">=":"gte",
    "!=":"not"
}

TRUE_VALUES = ['1', 'Y', 'y', 'yes', 'True', 'true']


def filter(qset, query_string, search_fields, value_fields=[]):
    fq = get_query(query_string, search_fields, value_fields)
    if fq:
        qset = qset.filter(fq)
    sq = get_query(query_string, search_fields, value_fields, excludes=True)
    if sq:
        qset = qset.exclude(sq) 
    return qset

def parseComparison(term):
    comp = None
    comparators = [e for e in SEARCH_COMPARES if e in term]
    if not comparators:
        return None, None, None
    comp = comparators[0]
    dfilter = COMPARATOR_TO_FILTER.get(comp)
    try:
        field, sterm = term.split(comp)
        return field, sterm, dfilter
    except:
        pass
    return None, None, None

def normalize_query(query_string,
                    findterms=re.compile(r'"([^"]+)"|(\S+)').findall,
                    normspace=re.compile(r'\s{2,}').sub):
    ''' Splits the query string in invidual keywords, getting rid of unecessary spaces
        and grouping quoted words together.
        Example:

        >>> normalize_query('  some random  words "with   quotes  " and   spaces')
        ['some', 'random', 'words', 'with quotes', 'and', 'spaces']

    '''
    return [normspace(' ', (t[0] or t[1]).strip()) for t in findterms(query_string)]

def get_properties_search(fname, fields, tfilter, tvalue):
    key = fields[1]
    if "." in key:
        # examples "properties|. would be properties(key=tvalue, value__in=[1, 'y', 'Y', 'True', 'true'])"
        # examples "properties|permissions. would be properties(category="permissions", key=tvalue, value__in=[1, 'y', 'Y', 'True', 'true'])"
        category, key = key.split('.')
        if not category:
            # this means there is no category and the value is the key
            category = None
        if not key:
            # this means the value is the key
            key = tvalue
            qq = {"properties__key":key, "properties__category":category, "properties__value__in": TRUE_VALUES}
        else:
            qq = {"properties__key":key, "properties__category":category, "properties__value__{}".format(tfilter): tvalue}
    else:
        qq = {"properties__key":key, "properties__category":None, "properties__value__{}".format(tfilter): tvalue}
    q = Q(**qq)
    return q

def get_datetime_query(fname, fields, tfilter, tvalue):
    try:
        if isinstance(tvalue, str):
            if tvalue[-1] in ["h", "d", "m", "y"]:
                span = tvalue[-1]
                value = int(tvalue[:-1])
                if span == "h":
                    tvalue = datetime.now() + timedelta(hours=value)
                elif span == "d":
                    tvalue = datetime.now() + timedelta(days=value)
                elif span == "m":
                    tvalue = datetime.now() + timedelta(months=value)
                elif span == "y":
                    tvalue = datetime.now() + timedelta(years=value)
            else:
                tvalue = parseDateTime(tvalue)
        if tvalue is None:
            return None
        fname = fields[0]
        if tfilter == "icontains":
            tfilter = "gte"
        return Q(**{"{0}__{1}".format(fname, tfilter): tvalue})
    except Exception:
        pass
    return None

def get_query(query_string, search_fields, value_fields=[], default_filter="icontains", excludes=False):
    ''' Returns a query, that is a combination of Q objects. That combination
        aims to search keywords within a model by testing the given search fields.

    '''
    helpers.log_print(search_fields)
    query = None # Query to search for every search term
    terms = normalize_query(query_string)
    # copy search fields
    if search_fields is None:
        search_fields = []
    search_fields = search_fields[:]
    value_fields_keys = []
    value_fields_map = {}
    remove_terms = []
    for term in terms:
        or_query = None # Query to search for a given term in each field
        tfield, tvalue, tfilter = parseComparison(term)
        helpers.log_print("{}, {}, {}".format(tfield, tvalue, tfilter))
        if excludes:
            # only get excludes which is the "not"
            if tfilter != "not":
                continue
            tfilter = "icontains"
        if tfilter == "not":
            continue
        if tfilter:
            # we have a comparison term being used
            if value_fields and not value_fields_keys:
                # lets build a mapping for keys with simple terms like last_name == account__last_name
                for key in value_fields:
                    if isinstance(key, tuple):
                        value_fields_keys.append(key[0])
                        value_fields_map[key[0]] = key[1]
                    else:
                        value_fields_keys.append(key)
                        value_fields_map[key] = key
            if not tfield:
                # we need to search all fields
                for field_name in value_fields:
                    fname = field_name
                    if isinstance(field_name, tuple):
                        fname = field_name[0]
                    q = Q(**{"{0}__{1}".format(fname, tfilter): tvalue})
                    if or_query is None:
                        or_query = q
                    else:
                        or_query = or_query | q
            elif tfield in search_fields:
                q = Q(**{"{0}__{1}".format(tfield, tfilter): tvalue})
            elif tfield in value_fields_keys:
                # now we need to find the
                fname = value_fields_map[tfield]
                if "|" in fname:
                    # this is a special type
                    fields = fname.split('|')
                    if fields[0] == "properties":
                        q = get_properties_search(fname, fields, tfilter, tvalue)
                    else:
                        continue
                elif "#" in fname:
                    fields = fname.split('#')
                    if fields[1] == "datetime":
                        q = get_datetime_query(fname, fields, tfilter, tvalue)
                        if not q:
                            continue
                    else:
                        continue
                else:
                    q = Q(**{"{0}__{1}".format(fname, tfilter): tvalue})
            else:
                # field is not supported in search
                remove_terms.append(term)
                helpers.log_print("field: '{}'' not supported in search".format(tfield), value_fields_keys)
                continue
            if or_query is None:
                or_query = q
            else:
                or_query = or_query | q
        elif search_fields:
            # simple search so lets search all fields
            for field_name in search_fields:
                q = Q(**{"{0}__{1}".format(field_name, default_filter): term})
                if or_query is None:
                    or_query = q
                else:
                    or_query = or_query | q
        if query is None:
            query = or_query
        else:
            query = query | or_query
    terms = [x for x in terms if x not in remove_terms]
    return query





from . import ffmpeg
from .. import render_utils
import copy

PresetConfig = [{
    'name':		'Flash Video',
    'short_name':	'flv',
    'module_name':	'flv',
    'description':	'Flash x264/AAC video',
    'stage':	190,
    'default_use':	'video',
    'applicable_kinds': 'VLAS',
    'parameters': {
    },
}]
PresetConfig[0]['parameters'].update(copy.deepcopy(ffmpeg.PresetParameters))
PresetConfig[0]['parameters'].update(copy.deepcopy(render_utils.PresetParameters))
render_utils.update_params(PresetConfig[0], 
    audio_bitrate=128,
    audio_channels= 2,
    audio_freq= 44100,
    video_bitrate= 512,
    frame_rate= 30,
    key_frame=25,
    __pass= 2,
)

PresetConfig.append(copy.deepcopy(PresetConfig[0]))
PresetConfig[1].update({
    'name':		'Flash Video (720p)',
    'short_name':	'flv_hq',
    'module_name':	'flv',
    'description':	'Flash x264/AAC video (720p)',
})
render_utils.update_params(PresetConfig[1],
    frame_rate= 25,
    video_bitrate= 1200,
    audio_bitrate=168,
    height=720,
    key_frame=16,
)


PresetConfig.append(copy.deepcopy(PresetConfig[0]))
PresetConfig[2].update({
    'name':		'Flash Video (480p)',
    'short_name':	'flv_mq',
    'module_name':	'flv',
    'description':	'Flash x264/AAC video (360p)',
})
render_utils.update_params(PresetConfig[2],
    frame_rate= 25,
    video_bitrate= 512,
    audio_bitrate=96,
    height=480,
    key_frame=25,
)

PresetConfig.append(copy.deepcopy(PresetConfig[0]))
PresetConfig[3].update({
    'name':		'Flash Video (360p)',
    'short_name':	'flv_lq',
    'module_name':	'flv',
    'description':	'Flash x264/AAC video (360p)',
})
render_utils.update_params(PresetConfig[2],
    frame_rate= 15,
    video_bitrate= 128,
    audio_bitrate=40,
    height=360,
    key_frame=30,
)


def render(item, renditiondef, params):
    params['video_container'] = 'flv'
    params['video_codec'] = 'libx264'
    params['audio_codec'] = 'libfaac'
    if params.get('audio_bitrate') >= 96 and 'audio_channels' not in params:
        params['audio_channels'] = 2
    return ffmpeg.render(item, renditiondef, params)
